# type: ignore
"""
A Flake8 Plugin for use in Globus Compute

Catches log calls which do eager string formatting and imports of the `typing`
module which don't match our preferred form.
"""

import ast

CODEMAP = {
    "FX001": "FX001 log call uses string.format() on first arg",
    "FX002": "FX002 log call uses string.join() on first arg",
    "FX003": "FX003 log call uses %-formatting in first arg",
    "FX004": "FX004 log call uses '+' operator in first arg",
    "FX005": "FX005 log call uses f-string in first arg",
    "FX010": "FX010 bare import of typing module",
    "FX011": "FX011 import of typing module with an alias other than 't'",
}
LOG_METHODS = {"debug", "info", "warn", "warning", "error", "critical", "exception"}


class Plugin:
    name = "globus-compute-flake8"
    version = "0.1.0"

    # args to init determine plugin behavior. see:
    # https://flake8.pycqa.org/en/latest/internal/utils.html#flake8.utils.parameters_for
    def __init__(self, tree):
        self.tree = tree

    # Plugin.run() is how checks will run. For detail, see implementation of:
    # https://flake8.pycqa.org/en/latest/internal/checker.html#flake8.checker.FileChecker.run_ast_checks
    def run(self):
        visitors = (LogMethodVisitor(), ImportVisitor())
        for visitor in visitors:
            visitor.visit(self.tree)
            for lineno, col, code in visitor.collect:
                yield lineno, col, CODEMAP[code], type(self)


class ErrorRecordingVisitor(ast.NodeVisitor):
    def __init__(self):
        super().__init__()
        self.collect = []

    def _record(self, node, code):
        self.collect.append((node.lineno, node.col_offset, code))


class LogMethodVisitor(ErrorRecordingVisitor):
    def __init__(self):
        super().__init__()
        self.in_call = False
        self.in_firstarg = False

    def visit_Call(self, node):
        try:
            current_func = node.func.attr
        except AttributeError:
            self.generic_visit(node)
            return

        # in a log call, in the first arg of the call, in a call to format()
        if self.in_call and self.in_firstarg:
            if current_func == "format":
                self._record(node, "FX001")
                self.generic_visit(node)
                return
            elif current_func == "join":
                self._record(node, "FX002")
                self.generic_visit(node)
                return

        if current_func not in LOG_METHODS:
            self.generic_visit(node)
            return

        self.in_call = True

        for i, child in enumerate(ast.iter_child_nodes(node)):
            if i == 1:
                self.in_firstarg = True
            self.visit(child)
            self.in_firstarg = False

        self.in_call = False

    def visit_BinOp(self, node):
        if self.in_call and self.in_firstarg:
            if isinstance(node.op, ast.Mod):
                self._record(node, "FX003")
                self.generic_visit(node)
            elif isinstance(node.op, ast.Add):
                self._record(node, "FX004")
                self.generic_visit(node)

    def visit_JoinedStr(self, node):
        if self.in_call and self.in_firstarg:
            self._record(node, "FX005")
            self.generic_visit(node)


class ImportVisitor(ErrorRecordingVisitor):
    def visit_Import(self, node):  # an `import foo` clause
        for alias in node.names:
            if alias.name == "typing":
                if alias.asname is None:
                    self._record(node, "FX010")
                elif alias.asname != "t":
                    self._record(node, "FX011")



import typing as t
import uuid

from .base import Message, meta
from .container import Container


@meta(message_type="task")
class Task(Message):
    task_id: uuid.UUID

    # for backward compatibility; to be removed when we support only
    # endpoints >= v1.1
    container_id: t.Optional[uuid.UUID]

    container: t.Optional[Container]
    task_buffer: str



import typing as t

from ...tasks.constants import ActorName, TaskState
from .base import Message, meta


@meta(message_type="task_transition")
class TaskTransition(Message):
    timestamp: int
    state: TaskState
    actor: ActorName

    def to_dict(self) -> t.Dict[str, t.Any]:
        return {
            "timestamp": self.timestamp,
            "state": self.state.value,
            "actor": self.actor.value,
        }



import typing as t
from datetime import datetime

from .redis import FLOAT_SERDE, HasRedisFieldsMeta, RedisField

try:
    import redis

    has_redis = True
except ImportError:
    has_redis = False


class RedisEndpointLock(metaclass=HasRedisFieldsMeta):
    """
    ORM-esque class to wrap access to a lock on an endpoint UUID

    Creation:
      Create a lock by instantiating this class, e.g.
      >>> RedisEndpointLock(redis_client, "e_uuid_12345", 1665605117.077466)

    Finding:
      Returns a lock dict if the lock exists for the endpoint
      >>> RedisEndpointLock.find(redis_client, "e_uuid_12345")
      This will return None if there was no lock

    Checking for lock:
      Method that returns True if there exists a lock *now* for
      the specified endpoint UUID
      >>> RedisEndpointLock.exists(redis_client, "e_uuid_12345")

    This class provides various task fields as descriptors via RedisField, and is
    responsible for de/serializing various data from/to hstore in Redis.
    """

    endpoint_id = t.cast(str, RedisField())
    lock_expiration_ts = t.cast(float, RedisField(serde=FLOAT_SERDE))

    def __init__(
        self,
        redis_client: "redis.Redis[t.Any]",
        endpoint_id: str,
        *,
        lock_timestamp: t.Optional[float] = None,
    ):
        """
        :param Redis redis_client: Redis client for properties to get/set
        :param str endpoint_id: UUID of the task, as str
        :param float lock_timestamp: The epoch timestamp when the lock ends
        """
        # non-RedisField attributes of a RedisEndpointLock
        self.hname = f"endpoint_lock_{endpoint_id}"
        self.redis_client = redis_client
        self.endpoint_id = endpoint_id
        if lock_timestamp is not None:
            now_ts = datetime.now().timestamp()
            if lock_timestamp <= now_ts:
                raise ValueError("Requested lock timestamp must be in the future")

            # This is stored as a float in the data, for our usage
            self.lock_expiration_ts = lock_timestamp
            # expire_ts is the internal reddit expiry, millisecond precision
            self.expire_ts = lock_timestamp

    @property
    def expire_ts(self) -> float:
        # TODO replace with pexpiretime() on upgrade
        # pexpiretime() should return what we need but it's apparently not
        # in the version of redis library we use?
        # Note that this is ms precision as pexpireat setter is ms
        now_ts = datetime.now().timestamp()
        return now_ts + self.redis_client.pttl(self.hname) / 1000

    @expire_ts.setter
    def expire_ts(self, expiration_ts: float) -> None:
        """Expires lock after expiration time, if not already set."""
        ttl_ms = self.redis_client.pttl(self.hname)
        if ttl_ms < 0:
            expiration_ts_ms = int(expiration_ts * 1000)
            self.redis_client.pexpireat(self.hname, expiration_ts_ms)

    def delete(self) -> None:
        """Removes this lock from Redis, not used for initial expiring case"""
        self.redis_client.delete(self.hname)

    @classmethod
    def exists(cls, redis_client: "redis.Redis[t.Any]", endpoint_id: str) -> bool:
        """Check if a given lock exists"""
        return bool(redis_client.exists(f"endpoint_lock_{endpoint_id}"))

    @classmethod
    def load(
        cls, redis_client: "redis.Redis[t.Any]", endpoint_id: str
    ) -> "RedisEndpointLock":
        """
        Returns an object lock if it exists, otherwise raises
        """
        if cls.exists(redis_client, endpoint_id):
            return cls(redis_client, endpoint_id)
        raise ValueError(f"Cannot find lock {endpoint_id}")



"""
The python protocol definition.

A MessagePackProtocol only needs to define two actions:
  - pack()
  - unpack()

And the two must be inverses on any valid inputs.
"""

import abc

from .message_types import Message


class MessagePackProtocol(abc.ABC):
    @abc.abstractmethod
    def pack(self, message: Message) -> bytes:
        """
        Pack a message into bytes.
        """

    @abc.abstractmethod
    def unpack(self, buf: bytes) -> Message:
        """
        Unpack bytes into a message.
        """



class InvalidMessageError(ValueError):
    """
    Message contents did not pass validation checks.

    This can be raised during unpacking OR when creating a new message.
    """


class UnrecognizedProtocolVersion(InvalidMessageError):
    """
    When attempting to unpack a message, found a protocol version which was not
    supported.
    """


class WrongMessageTypeError(InvalidMessageError):
    """
    Raised when a message has its type asserted and the type does not match the code's
    expectations.
    """



import typing as t

from .base import Message
from .container import Container, ContainerImage
from .ep_status_report import EPStatusReport
from .manager_status_report import ManagerStatusReport
from .result import Result, ResultErrorDetails
from .task import Task
from .task_cancel import TaskCancel
from .task_transition import TaskTransition

ALL_MESSAGE_CLASSES: t.Set[t.Type[Message]] = {
    Container,
    ContainerImage,
    EPStatusReport,
    ManagerStatusReport,
    Task,
    TaskCancel,
    Result,
    TaskTransition,
}

__all__ = (
    "Message",
    "Container",
    "ContainerImage",
    "EPStatusReport",
    "ManagerStatusReport",
    "Task",
    "TaskCancel",
    "Result",
    "ResultErrorDetails",
    "TaskTransition",
    "ALL_MESSAGE_CLASSES",
)



import typing as t

from .base import Message, meta
from .task_transition import TaskTransition


@meta(message_type="manager_status_report")
class ManagerStatusReport(Message):
    """
    Status report sent from the Manager to the Endpoint, which mostly just amounts to
    saying which tasks are now RUNNING.
    """

    task_statuses: t.Dict[str, t.List[TaskTransition]]



import typing as t
import uuid

from globus_compute_common.pydantic_v1 import Field

from .base import Message, meta
from .task_transition import TaskTransition


@meta(message_type="ep_status_report")
class EPStatusReport(Message):
    """
    Status report for an endpoint, sent from Endpoint to Forwarder.

    Includes EP-wide info such as utilization, as well as per-task status information.
    """

    endpoint_id: uuid.UUID
    global_state: t.Dict[str, t.Any] = Field(alias="ep_status_report")
    task_statuses: t.Dict[str, t.List[TaskTransition]]

    class Config:
        allow_population_by_field_name = True



import typing as t

from .constants import TaskState


# use of typing.Protocol requires python3.8
# since some parts of funcX support current pythons (py3.6, py3.8), we'll
# define the protocol as a normal class for simplicity for now
#
# TODO: evaluate conditional requirement for typing_extensions to allow the use
# of typing.Protocol
class TaskProtocol:
    task_id: str
    endpoint: t.Optional[str]
    status: TaskState
    result: t.Optional[str]
    result_reference: t.Optional[t.Dict[str, t.Any]]
    payload: t.Optional[str]
    payload_reference: t.Optional[t.Dict[str, t.Any]]



import typing as t
from enum import Enum

try:
    import boto3
    import botocore.exceptions

    has_boto3 = True
except ImportError:
    has_boto3 = False

from ..tasks import TaskProtocol
from .base import StorageException, TaskStorage
from .redis import ImplicitRedisStorage


class StorageFieldName(Enum):
    result = "result"
    payload = "payload"

    @property
    def reference_attr(self) -> str:
        return f"{self.value}_reference"


class RedisS3Storage(TaskStorage):
    """
    Abstract storage over Redis and S3.
    Uses Redis to store objects below of size threshold, and S3 for the rest
    """

    def __init__(self, *, bucket_name: str, redis_threshold: int) -> None:
        """
        :param bucket_name: Name of the S3 bucket to use
        :param redis_threshold: Max size(chars) of the data that redis accommodates
        """

        if not has_boto3:
            raise RuntimeError(
                "Cannot construct RedisS3Storage object since the boto3"
                "package is not available. Either install it explicitly or install the "
                "'boto3' extra, as in\n"
                "  pip install 'globus-compute-common[boto3]'"
            )

        self.bucket_name = bucket_name
        self.client = boto3.client("s3")

        self.redis_threshold = redis_threshold
        self.redis_storage = ImplicitRedisStorage()

    def _store_to_s3(
        self, task: TaskProtocol, storage_field_name: StorageFieldName, result: str
    ) -> None:
        key = f"{task.task_id}.{storage_field_name.reference_attr}"
        try:
            self.client.put_object(
                Body=result.encode("utf-8"),
                Bucket=self.bucket_name,
                Key=key,
            )
        except botocore.exceptions.ClientError as err:
            raise StorageException(
                f"Putting {storage_field_name.reference_attr} into s3 for "
                f"task:{task.task_id} failed"
            ) from err
        else:
            reference = {
                "storage_id": "s3",
                "s3bucket": self.bucket_name,
                "key": key,
            }
            setattr(task, storage_field_name.reference_attr, reference)

    def _get_from_s3(
        self, task: TaskProtocol, storage_field_name: StorageFieldName
    ) -> str:
        reference = getattr(task, storage_field_name.reference_attr)
        if reference is None:  # pragma: no cover
            raise StorageException(
                f"task {task.task_id} result reference was None inside of _get_from_s3"
            )
        try:
            bucket = reference["s3bucket"]
            key = reference["key"]
        except KeyError as err:
            raise StorageException(
                f"task {task.task_id} {storage_field_name.reference_attr} "
                "pointed to S3, but was missing s3bucket or key"
            ) from err

        if not isinstance(bucket, str):
            raise StorageException(
                f"task {task.task_id} {storage_field_name.reference_attr} "
                f"pointed to S3, but s3bucket was of type {type(bucket)}, "
                "(expected string)"
            )

        if not isinstance(key, str):
            raise StorageException(
                f"task {task.task_id} {storage_field_name.reference_attr} "
                f"pointed to S3, but key was of type {type(key)} (expected string)"
            )

        try:
            response = self.client.get_object(Bucket=bucket, Key=key)
        except botocore.exceptions.ClientError as err:
            raise StorageException(
                f"Fetching object from S3 failed for: {task.task_id}"
            ) from err
        body = response["Body"]
        return t.cast(str, body.read().decode("utf-8"))

    def store_result(
        self,
        task: TaskProtocol,
        result: str,
    ) -> None:
        if len(result) > self.redis_threshold:
            # Task is too big for Redis, store in S3
            self._store_to_s3(task, StorageFieldName.result, result)
        else:
            self.redis_storage.store_result(task, result)

    def get_result(self, task: TaskProtocol) -> t.Optional[str]:
        """

        :param task:
        :return: Results result if available, else returns None
        Raises StorageException if fetching fails
        """
        # We should be able to safely remove the following block
        # once all tasks launched with v0.3.3 and prior have TTL'ed out
        if task.result:
            return task.result

        if task.result_reference:
            if task.result_reference["storage_id"] == "s3":
                return self._get_from_s3(task, StorageFieldName.result)

            # The following is redundant now while the block above
            # for backward compat exists
            # remove the pragma once this is an active codepath
            elif task.result_reference["storage_id"] == "redis":  # pragma: no cover
                return self.redis_storage.get_result(task)
            else:
                raise StorageException(
                    f"Unknown Storage requested: {task.result_reference}"
                )
        else:
            return None

    def store_payload(self, task: TaskProtocol, payload: str) -> None:
        if len(payload) > self.redis_threshold:
            # Task is too big for Redis, store in S3
            self._store_to_s3(task, StorageFieldName.payload, payload)
        else:
            self.redis_storage.store_payload(task, payload)

    def get_payload(self, task: TaskProtocol) -> t.Optional[str]:
        """

        :param task:
        :return: Results payload if available, else returns None
        Raises StorageException if fetching fails
        """
        # We should be able to safely remove the following block
        # once all tasks launched with v0.3.3 and prior have TTL'ed out
        if task.payload:
            return task.payload

        if task.payload_reference:
            if task.payload_reference["storage_id"] == "s3":
                return self._get_from_s3(task, StorageFieldName.payload)

            # The following is redundant now while the block above
            # for backward compat exists
            # remove the pragma once this is an active codepath
            elif task.payload_reference["storage_id"] == "redis":  # pragma: no cover
                return self.redis_storage.get_payload(task)
            else:
                raise StorageException(
                    f"Unknown Storage requested: {task.payload_reference}"
                )
        else:
            return None



import typing as t

from .serde import DEFAULT_SERDE, ComputeRedisSerde

_null_key = "__NULL_KEY__"


class RedisField:
    """
    Descriptor class that stores data in redis.

    Uses owning class's redis client in `owner.redis_client` to connect, and uses
    owner's hname in `owner.hname` to uniquely identify the keys.

    Fields can be serialized and deserialized by setting a ComputeRedisSerde.
    """

    # TODO: have a cache and TTL on the properties so that we aren't making so many
    #       redis gets?
    def __init__(self, serde: ComputeRedisSerde = DEFAULT_SERDE) -> None:
        self.serde = serde
        self.key: str = _null_key  # will be overwritten

    def _check_null_key(self) -> None:
        if self.key == "__NULL_KEY__":
            raise TypeError(
                "Cannot use RedisField outside of a class with "
                "HasRedisFieldsMeta. Inherit from HasRedisFields or set "
                "metaclass to HasRedisFieldsMeta."
            )

    def __get__(self, owner: t.Any, ownertype: t.Type[t.Any]) -> t.Any:
        self._check_null_key()
        value = owner.redis_client.hget(owner.hname, self.key)
        return None if value is None else self.serde.deserialize(value)

    def __set__(self, owner: t.Any, val: t.Any) -> None:
        self._check_null_key()
        owner.redis_client.hset(owner.hname, self.key, self.serde.serialize(val))


class HasRedisFieldsMeta(type):
    """
    This metaclass should be used by any class which has RedisFields included.
    It is also provided via the HasRedisFields class for use in simple
    inheritance, for convenience.

    This inspects all class attributes and sets the keys on RedisField
    attributes to be the same as their attribute name.
    """

    # don't type check __new__ -- metaclasses are hard for mypy
    def __new__(mcls, classname, bases, class_attrs):  # type: ignore
        for attrname, value in class_attrs.items():
            if isinstance(value, RedisField):
                value.key = attrname
        return super().__new__(mcls, classname, bases, class_attrs)


class HasRedisFields(metaclass=HasRedisFieldsMeta):
    pass



"""
This file defines Protocol Version 1 of the funcx messagepack protocol.

v1 of the protocol does the following:
- each message is encoded as a JSON object in UTF-8 with a one-byte header, no newlines
- the first byte of the header contains the protocol version, always 1
- the JSON body contains exactly two fields: "message_type" (a string) and "data" (an object)
- "message_type" determines which class is used to load the content in "data"

For example, under protocol v1, a heartbeat request message could be formulated as

    \x01{"message_type":"heartbeat_req","data":{}}

where `\x01` is the version byte.

A Task message can be formulated with

    \x01{"message_type":"task","data":{"task_id": "359e2ec3-caef-4258-a149-a67767af0ee8","container_id":"ee138b83-fd62-4b57-b31b-55e460569fd2"}}

== Multi-Message Payloads

Because newlines are a forbidden character, multiple messages may be newline-delimited, as in

    \x01{"message_type":"foo","data":{}}
    \x01{"message_type":"bar","data":{}}

== Unknown Field Handling (loading)

The protocol defines the following behavior for missing and unknown fields:

  - if a payload does not contain a required field, loading will fail with a
    pydantic ValidationError

  - if a payload defines fields which are not included in the current message
    definitions, they are ignored and a warning is logged

  - if an envelope includes unknown fields (other than "message_type" and "data"), they
    are ignored and a warning is logged

Therefore, new fields may be added to the message classes, but they must not be
required until all messages contain those fields.
"""  # noqa: E501

from __future__ import annotations

import json
import logging
import typing as t

from globus_compute_common import pydantic_v1

from ..message_types import ALL_MESSAGE_CLASSES, Message
from ..protocol import MessagePackProtocol

log = logging.getLogger(__name__)

# protocol version and reserved byte as a byte array
_VERSION_BYTE = (1).to_bytes(1, byteorder="big", signed=False)

# internal to this module, a mapping from message classes to their names
_MESSAGE_TYPE_MAP: dict[str, type[Message]] = {
    m.Meta.message_type: m for m in ALL_MESSAGE_CLASSES
}


class MessageEnvelope(pydantic_v1.BaseModel):
    message_type: str
    data: t.Dict[str, t.Any]

    @pydantic_v1.validator("message_type")
    def message_type_is_known(cls, v: str) -> str:
        if v not in _MESSAGE_TYPE_MAP:
            # pydantic will wrap this message + context in a ValidationError
            raise ValueError("unrecognized value")
        return v


_ModelT = t.TypeVar("_ModelT", bound=t.Union[Message, MessageEnvelope])


def _log_unknown_fields(model: type[_ModelT], data: dict[str, t.Any]) -> None:
    if issubclass(model, MessageEnvelope):
        outer_type = "envelope"
        message_type = data["message_type"]
    elif issubclass(model, Message):
        outer_type = "data"
        message_type = model.Meta.message_type
    else:
        raise NotImplementedError

    model_ = t.cast(pydantic_v1.BaseModel, model)

    unknown_fields = set(data)
    for name, field in model_.__fields__.items():
        unknown_fields.discard(name)
        unknown_fields.discard(field.alias)

    if unknown_fields:
        log.warning(
            "encountered unknown %s fields while reading a %s message: %s",
            outer_type,
            message_type,
            unknown_fields,
        )


def _load(model: type[_ModelT], data: dict[str, t.Any]) -> _ModelT:
    model_ = t.cast("type[pydantic_v1.BaseModel]", model)
    ret = model_.parse_obj(data)
    _log_unknown_fields(model, data)
    return t.cast(_ModelT, ret)


class MessagePackProtocolV1(MessagePackProtocol):
    def pack(self, message: Message) -> bytes:
        body = MessageEnvelope(message_type=message.message_type, data=message.dict())
        # encode() defaults to UTF-8, which is what the protocol specifies
        return _VERSION_BYTE + body.json(separators=(",", ":")).encode()

    def unpack(self, buf: bytes) -> Message:
        # strip the version byte header
        body = buf[1:]
        payload = json.loads(body)
        envelope = _load(MessageEnvelope, payload)

        message_class = _MESSAGE_TYPE_MAP[envelope.message_type]
        message = _load(message_class, envelope.data)
        return message



import os
import typing as t

from .base import TaskStorage
from .redis import ImplicitRedisStorage
from .s3 import RedisS3Storage

DEFAULT_REDIS_STORAGE_THRESHOLD: int = 20000


def _get_redis_storage_threshold() -> int:
    val = os.getenv("COMPUTE_REDIS_STORAGE_THRESHOLD")
    if val is not None:
        try:
            return int(val)
        except ValueError as err:
            raise ValueError(
                f"could not parse COMPUTE_REDIS_STORAGE_THRESHOLD={val} as int"
            ) from err
    return DEFAULT_REDIS_STORAGE_THRESHOLD


def _s3_bucket_name() -> t.Optional[str]:
    return os.getenv("COMPUTE_S3_BUCKET_NAME")


def get_default_task_storage() -> TaskStorage:
    bucket = _s3_bucket_name()
    threshold = _get_redis_storage_threshold()

    # a redis threshold of -1 means "never use S3, just use Redis"
    if bucket is None or threshold == -1:
        return ImplicitRedisStorage()
    else:
        return RedisS3Storage(
            bucket_name=bucket, redis_threshold=_get_redis_storage_threshold()
        )



from __future__ import annotations

import typing as t

from globus_compute_common.pydantic_v1 import BaseModel

from ..exceptions import WrongMessageTypeError

MT = t.TypeVar("MT", bound=t.Type["Message"])


class Message(BaseModel):
    # pydantic inspects most data on the class itself
    # use an internal class to declare attributes of the message class which should not
    # be treated by pydantic as part of the model
    class Meta:
        message_type: t.ClassVar[str]

    @property
    def message_type(self) -> str:
        return self.Meta.message_type

    # common Config for all of our pydantic models
    class Config:
        # set this flag to allow underscore-prefixed attrs to be used rather than
        # pydantic.PrivateAttr to declare instance variables on models which are
        # not part of the serialized data
        # see:
        #   https://pydantic-docs.helpmanual.io/usage/models/#private-model-attributes
        underscore_attrs_are_private = True

    def assert_one_of_types(self, *message_types: type[Message]) -> None:
        if not isinstance(self, message_types):
            raise WrongMessageTypeError(
                f"expected {message_types} but got {type(self)}"
            )


# a handy class decorator for assigning fields to the internal Meta class
# correctly subclasses any Meta which might be defined on a message type
def meta(**kwargs: t.Any) -> t.Callable[[MT], MT]:
    def class_decorator(cls: MT) -> MT:
        if not hasattr(cls, "Meta"):
            bases: tuple[type[t.Any], ...] = ()
        else:
            bases = (cls.Meta,)

        # this is incomprehensible magic to mypy, just type-ignore it
        cls.Meta = type("Meta", bases, {})  # type: ignore

        for k, v in kwargs.items():
            setattr(cls.Meta, k, v)

        return cls

    return class_decorator



"""Pydantic V2 provides access to the V1 API, enabling us to
continue using V1 while also supporting V2.
"""

try:
    from pydantic.v1 import *  # noqa: F401 F403
except ImportError:
    from pydantic import *  # type: ignore # noqa: F401 F403
